/*--------------------------------------------------
HTTP 1.1 Webserver for ESP8266 
for ESP8266 adapted Arduino IDE
--------------------------------------------------*/

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>

const char* ssid = "Hier-muss-die-WLAN-SSID-rein";
const char* password = "xxxxxxxxxxxxxxxxxxxxxx";

const int PIN_RELAIS_1 = 5;
const int PIN_RELAIS_2 = 4;
const int PIN_RELAIS_3 = 14;
const int PIN_RELAIS_4 = 12;

unsigned long ulReqcount;
unsigned long ulReconncount;


// Create an instance of the server on Port 80
ESP8266WebServer server(80);

void setup() 
{
  // setup globals
  ulReqcount=0; 
  ulReconncount=0;
  
  // GPIOs Konfigurieren
  pinMode(PIN_RELAIS_1, OUTPUT);
  pinMode(PIN_RELAIS_2, OUTPUT);
  pinMode(PIN_RELAIS_3, OUTPUT);
  pinMode(PIN_RELAIS_4, OUTPUT);

  // Alle Relais Aus
  digitalWrite(PIN_RELAIS_1, 1);
  digitalWrite(PIN_RELAIS_2, 1);
  digitalWrite(PIN_RELAIS_3, 1);
  digitalWrite(PIN_RELAIS_4, 1);
  
  // start serial
  Serial.begin(9600);
  delay(1);
  
  // inital connect
  WiFi.mode(WIFI_STA);
  WiFiStart();

  // Setup server routes 
  server.on("/", handle_index);
  server.on("/api", handle_api);

  // Start the server 
  server.begin();
  Serial.println("HTTP server started");  
}

void loop() 
{
  // check if WLAN is connected
  if (WiFi.status() != WL_CONNECTED)
    WiFiStart();

  // check for incomming client connections frequently in the main loop
  server.handleClient();
}

/* ------------------------------------------------------------------
 * Return the Homepage
 * ------------------------------------------------------------------
 */
void handle_index() {
  server.send(200, "text/plain", "This is an index page.");
}

/* ------------------------------------------------------------------
 * Handle API-Requests
 * ------------------------------------------------------------------
 */
void handle_api() {
  int motor = 0;
  if (server.arg("motor").equalsIgnoreCase("top"))
    motor = 1;
  else if (server.arg("motor").equalsIgnoreCase("bottom"))
    motor = 2;
  else 
  {
    server.send(500, "text/plain", String("Parameter 'motor' must be TOP or BOTTOM"));
    return;
  }

  int state = 0;
  if (server.arg("state").equalsIgnoreCase("UP"))
    state = 1;
  else if (server.arg("state").equalsIgnoreCase("DOWN"))
    state = 2;
  
  switch(motor)
  {
    case 1:
      if (state == 1) MotorObenUp();
      else if (state == 2) MotorObenDown();
      else MotorObenStop();
      server.send(200, "text/plain", String("OK"));
      return;
    case 2:
      if (state == 1) MotorUntenUp();
      else if (state == 2) MotorUntenDown();
      else MotorUntenStop();
      server.send(200, "text/plain", String("OK"));
      return;
  }

  server.send(500, "text/plain", String("Unexpected error."));
}

void MotorObenUp()
{
  digitalWrite(PIN_RELAIS_2, 1);
  digitalWrite(PIN_RELAIS_1, 0);
}

void MotorObenDown()
{
  digitalWrite(PIN_RELAIS_2, 0);
  digitalWrite(PIN_RELAIS_1, 0);
}

void MotorObenStop()
{
  digitalWrite(PIN_RELAIS_1, 1);
  digitalWrite(PIN_RELAIS_2, 1);
}

void MotorUntenUp()
{
  digitalWrite(PIN_RELAIS_4, 1);
  digitalWrite(PIN_RELAIS_3, 0);
}

void MotorUntenDown()
{
  digitalWrite(PIN_RELAIS_4, 0);
  digitalWrite(PIN_RELAIS_3, 0);
}

void MotorUntenStop()
{
  digitalWrite(PIN_RELAIS_3, 1);
  digitalWrite(PIN_RELAIS_4, 1);
}

/* ------------------------------------------------------------------
 * Start the WiFi Connection
 * ------------------------------------------------------------------
 */
void WiFiStart()
{
  ulReconncount++;
  
  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  
  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());
}
